(defvar bootstrap-version)
(let ((bootstrap-file
       (expand-file-name "straight/repos/straight.el/bootstrap.el" user-emacs-directory))
      (bootstrap-version 5))
  (unless (file-exists-p bootstrap-file)
    (with-current-buffer
        (url-retrieve-synchronously
         "https://raw.githubusercontent.com/raxod502/straight.el/develop/install.el"
         'silent 'inhibit-cookies)
      (goto-char (point-max))
      (eval-print-last-sexp)))
  (load bootstrap-file nil 'nomessage))

(straight-use-package 'use-package)
(use-package straight
  :custom (straight-use-package-by-default t))

(menu-bar-mode -1)
(when window-system
  (tool-bar-mode -1)
  (scroll-bar-mode -1))

(setq backup-directory-alist '(("." . "~/.emacs.d/autosave")))

(column-number-mode)
(setq inhibit-splash-screen t
      split-height-threshold 80
      split-width-threshold 160
      display-time-default-load-average nil
      display-time-24hr-format t
      fill-column 90)
(display-time-mode t)
(delete 'display-time-string global-mode-string)
(setq custom-file "~/.config/emacs-custom.el")
(load custom-file)
(customize-set-variable 'mode-line-format
                        '("%e"
                          display-time-string
                          mode-line-modified
                          mode-line-remote
                          mode-line-buffer-identification
                          " "
                          mode-line-position
                          (vc-mode vc-mode)
                          " "
                          mode-line-modes
                          mode-line-misc-info))
(use-package markdown-mode)
(use-package yaml-mode)

(defcustom theme-shuffler-idle-timer-interval (* 60 60)
  "Time interval in seconds for theme-shuffler idle timer")

(defvar theme-shuffler--current-theme nil "Current theme")

(defvar theme-shuffler--theme-list nil "Available themes")

(defmacro theme-shuffler-use-themes (&rest themes)
  "Add themes to theme shuffler."
  (let ((themes (reverse themes)))
    `(progn
       (setq theme-shuffler--theme-list nil)
       ,@(mapcar
          (lambda (theme)
            (let (package-name themes add-theme-form)
              (if (listp theme)
                  (setq package-name (car theme)
                        themes (cdr theme)
                        add-theme-form `(mapc
                                         (apply-partially 'add-to-list 'theme-shuffler--theme-list)
                                         ',themes))
                (setq package-name theme
                      add-theme-form `(add-to-list 'theme-shuffler--theme-list ',theme)))
              (setq package-name
                    (car (read-from-string (concat (symbol-name package-name) "-theme"))))
              `(progn (use-package ,package-name) ,add-theme-form)))
          themes))))

(defun theme-shuffler-change-theme (theme)
  "Disables all other themes and enable a new one"
  (interactive
   (list
    (intern (completing-read "Change to theme: "
                             (mapcar #'symbol-name
                                     theme-shuffler--theme-list)))))
  (mapc 'disable-theme custom-enabled-themes)
  (load-theme theme t)
  (setq theme-shuffler--current-theme theme)
  (message "Loaded theme: %s" (symbol-name theme)))

(defun theme-shuffler-shuffle-theme ()
  "Shuffles the currently loaded theme."
  (interactive)
  (theme-shuffler-change-theme
   (random-choice
    (remove theme-shuffler--current-theme theme-shuffler--theme-list))))

(defvar theme-shuffler--idle-timer nil)

(defun theme-shuffler-start-idle-timer ()
  "Start shuffler idle timer"
  (interactive)
  (message "Starting theme shuffler idle timer with %d second interval" theme-shuffler-idle-timer-interval)
  (setq theme-shuffler--idle-timer (run-with-idle-timer theme-shuffler-idle-timer-interval t #'shuffle-theme)))

(defun theme-shuffler-cancel-idle-timer ()
  "Cancel shuffler idle timer"
  (interactive)
  (if (not theme-shuffler--idle-timer)
      (message "No idle timer set")
    (message "Cancelling %d second theme shuffler idle timer" (elt theme-shuffler--idle-timer 2))
    (cancel-timer theme-shuffler--idle-timer)
    (setq theme-shuffler--idle-timer nil)))

(theme-shuffler-use-themes
 (ample ample ample-flat)
 anti-zenburn
 cyberpunk
 (darktooth darktooth-dark)
 dracula
 gruber-darker
 inkpot
 material
 monokai
 (solarized solarized-dark-high-contrast))

(defmacro when-user-file-exists (file-name &rest body)
  (declare (indent 1))
  `(let ((it ,(expand-file-name file-name user-emacs-directory)))
     (when (file-exists-p it)
       ,@body)))

(defun plistp (obj)
  "Non-nil if OBJ is a list with an even number of elements"
  (and (listp obj)
       (cl-evenp (length obj))))

(defun plist-keys (plist)
  "A list of keys in property list PLIST or nil if PLIST is not a property list"
  (if (plistp plist)
      (mapcar #'(lambda (i) (nth i plist))
              (number-sequence 0 (/ (length plist) 2) 2))))

(defun random-choice (lst)
  "Return random element of LST"
  (nth (random (length lst)) lst))

(defun unfill-paragraph ()
  (interactive)
  (let ((fill-column (point-max)))
    (fill-paragraph nil)))

(defun unfill-region ()
  (interactive)
  (let ((fill-column (point-max)))
    (fill-region (region-beginning) (region-end) nil)))

(defun read-file-to-string (filename)
  "Return the contents of file named `filename' as a string.

Why is this necessary for me to define? The world may never know."
  (with-temp-buffer
    (insert-file-contents filename)
    (buffer-string)))

(setq gnus-select-method '(nnimap "mail.hover.com")
      gnus-use-full-window nil)

(use-package helm
  :demand t
  :bind (("C-x b" . helm-mini)
         ("M-x" . helm-M-x)
         ("M-X" . execute-extended-command)
         ("C-h a" . helm-apropos)
         ("C-x C-f" . helm-find-files))
  :custom (helm-split-window-preferred-function split-window-preferred-function)
  :config (helm-mode 1))

(use-package org
  :bind (("C-c a" . org-agenda)
         ("C-c c" . org-capture)
         ("C-c C-x C-o" . org-clock-out)
         ("C-c C-x C-x" . org-clock-in-last)
         :map org-mode-map
         ("C-c C-t" . my/org-todo))
  :hook (org-mode . auto-revert-mode)
  :init
  (defvar org-main-file "~/org/core.org"
    "The single org file that I use for planning")
  (defun my/org-current-effective-time ()
    "If `org-use-effective-time' is T, prompt for date+time.
  
  Otherwise, use current time."
    (if org-use-effective-time
        (org-read-date nil t)
      (org-current-time)))
  (advice-add 'org-current-effective-time :override #'my/org-current-effective-time)
  
  (defun my/org-todo (arg)
    "Modify org-todo prefix arg behavior"
    (interactive "P")
    (let ((org-use-effective-time current-prefix-arg))
      (funcall #'org-todo nil)))
  :custom
  (org-agenda-files (list org-main-file))
  (org-agenda-file-regexp "\\.org$")
  (org-modules '(org-habit))
  (org-startup-folded t)
  (org-startup-indented t)
  (org-pretty-entities t)
  (org-hide-leading-stars t)
  (org-todo-repeat-to-state t)
  (org-log-into-drawer t)
  (org-blank-before-new-entry '((heading . nil)(plain-list-item . nil)))
  (org-refile-targets '((org-agenda-files . (:maxlevel . 4))))
  (org-agenda-restore-windows-after-quit t)
  (org-archive-location "%s_archive::datetree/")
  (org-archive-subtree-save-file-p t)
  (org-src-window-setup 'current-window)
  (org-todo-keywords '(;; tasks
                       (sequence "todo(t)" "|"
                                 "done(d!)" "notdone(n@)")
                       ;; projects
                       (sequence "future(f)" "inprogress(i!)" "waiting(w@)" "|"
                                 "completed(c!)" "canceled(q@)")
                       ;; events
                       (sequence "event(E)" "going(G!)" "|"
                                 "notgoing(N!)" "went(W@)" "skipped(S!)")))
  (org-capture-templates '(("i" "Generic inbox item"
                            entry (file+headline org-main-file "Inbox")
                            "* %?\n%U")
                           ("t" "Task"
                            entry (file+headline org-main-file "Inbox")
                            "* todo %?\n%U")
                           ("c" "Generic calendar item"
                            entry (file org-main-file)
                            "* %?\n%^t")))
  (org-agenda-custom-commands '(("d" "day agenda " agenda "" ((org-agenda-span 'day)))
                                ("w" "week agenda" agenda "")
                                ("m" "month agenda " agenda ""
                                 ((org-agenda-span 'month)
                                  (org-agenda-use-time-grid nil)))
                                ("t" "unscheduled todo list"
                                 todo "todo"
                                 ((org-agenda-todo-ignore-scheduled t)))
                                ("c" "completed tasks"
                                 todo "done|complete"
                                 ((org-agenda-todo-ignore-scheduled t))))))

(use-package format-all
  :hook (c-mode . format-all-mode))

(use-package magit
  :bind (("C-c m s" . magit-status)
         ("C-c m F" . magit-pull)
         ("C-c m P" . magit-push)
         ("C-c m r" . magit-rebase)
         ("C-c m c" . magit-clone)))

(add-to-list 'auto-mode-alist '("\\.dcl\\'" . c-mode))

(define-derived-mode autolisp-mode lisp-mode "AutoLISP"
  "Major mode for editing AutoLISP files"
  (set (make-local-variable 'indent-tabs-mode) nil)
  (slime-mode -1))
(add-hook 'autolisp-mode-hook (lambda() (slime-mode -1)))
(add-to-list 'auto-mode-alist '("\\.lsp\\'" . autolisp-mode))
(font-lock-add-keywords 'autolisp-mode
  '(
    ("setq" . font-lock-keyword-face)
    ("vl[a-zA-Z0-9>\\-]+" . font-lock-keyword-face)
    ("defun-q-list-ref" . font-lock-keyword-face)
    ("defun-q-list-set" . font-lock-keyword-face)
    ("declare-late" . font-lock-keyword-face)
    ("defun-q" . font-lock-keyword-face)
    ("defun-r" . font-lock-comment-face)
    ("defun-q[\n[:blank:]]+\\([*_<>:&a-zA-Z0-9#!\\-]+\\)" 1 font-lock-function-name-face)))
(add-hook 'autolisp-mode-hook (lambda() (setq fill-column 90)))

(use-package jinja2-mode
  :mode ("\\.jinja\\'" . jinja2-mode))

(use-package elpy
  :config (elpy-enable))

(use-package web-mode
  :mode ("\\.html\\'" . web-mode)
  :custom
  (web-mode-css-indent-offset 1)
  (web-mode-code-indent-offset 2)
  (web-mode-markup-indent-offset 1)
  (web-mode-engines-alist '(("django" . "\\.html\\'"))))

(use-package lua-mode)

(use-package projectile)
